#include <fstream>
#include <iostream>
#include <vector>
#include "Condition.h"
#include "DBEngine.h"

DBEngine::DBEngine(){
	size = 0;
}

void DBEngine::createTable(string n){
	Relation r(n);
	tables.push_back(r);
	size++;
}

void DBEngine::createTable(string n, vector<Attribute> a){
	Relation r(n, a);
	tables.push_back(r);
	size++;
}
	
void DBEngine::createTable(Relation r){ 
	tables.push_back(r); 
	size++;
}
	
vector<Relation> DBEngine::getRelations(){ 
	return tables; 
}

	
Relation& DBEngine::getTableFromName(string n){
	for(int i = 0; i < tables.size(); i++){
		if (tables[i].getTableName() == n){
			return tables[i];
		}
	}
}

//currently writes nothing meaningful
void DBEngine::saveToFile(vector<string> cmds){ 
	ofstream file;
	file.open("savefile.db");
	
	for(int i = 0; i < cmds.size(); ++i){
		file << cmds[i] << endl;
	}
	
	file.close();
}


Relation DBEngine::selection(string attName, string s, Relation r){
	equality(attName, s, r);
}

//assumes that all attribute titles are unique
Relation DBEngine::projection(vector<string> input, Relation r){
	
	vector<Attribute> v;
	string new_name = r.getTableName() + " Projection";
	
	for(int i = 0; i < input.size(); ++i) {
		
		for(int j = 0; j < r.getSize(); ++j) {
			if((r.getAttributes())[j].getName() == input[i])
				v.push_back((r.getAttributes())[j]);
		}
	}
	
	Relation temp(new_name, v);
	return temp;
}

//test error matching
void DBEngine::rename(Relation& r, vector<string> oldnames, vector<string> newnames){
	if (oldnames.size() != newnames.size()) {
		cout << "FAILURE TO RENAME: number of attributes do not match.\n";
		return;
	}
	
	else if (oldnames != r.getAttributeNames()) {
		cout << "FAILURE TO RENAME: the attributes to be renamed do not exist in the relation.\n";
		return;
	}
	
	else {
		for(int i = 0; i < oldnames.size(); ++i){
			r.renameAttribute(oldnames[i], newnames[i]);
		}
	}
}

Relation DBEngine::setUnion(Relation r1, Relation r2){
	if (r1.getAttributeNames() != r2.getAttributeNames()){
		cout << "Failure to union: the relations are not union-compatible.\nreturning the first relation.\n";
		return r1;
	}

	else {
		//currently all returned relations are called TEMP
		Relation new_r = r1;
		new_r.setTableName("TEMP");
		vector<string> temp;
		bool duplicate = false;

		for (int i = 0; i < r2.getAttributes()[0].getSize(); ++i) {
			temp = r2.getTuple(i);

			for (int j = 0; j < new_r.getAttributes()[0].getSize(); ++j){
				if (temp == new_r.getTuple(j)){
					duplicate = true;
					break;
				}
			}

			if (!duplicate) {
				new_r.insertTuple(temp);
			}

			duplicate = false;
		}

		return new_r;
	}
}

Relation DBEngine::setDiff(Relation r1, Relation r2){
	if (r1.getAttributeNames() != r2.getAttributeNames()){
		cout << "Failure to diff: the relations are not union-compatible.\nreturning the first relation.\n";
		return r1;
	}

	else {
		//currently all returned relations are called TEMP
		Relation new_r = r1;
		new_r.setTableName("TEMP");
		vector<string> temp;
		bool duplicate = false;

		for (int i = 0; i < r1.getAttributes()[0].getSize(); ++i){
			new_r.removeTuple(i);
		}

		for (int i = 0; i < r1.getAttributes()[0].getSize(); ++i) {
			temp = r1.getTuple(i);

			for (int j = 0; j < r2.getAttributes()[0].getSize(); ++j){
				if (temp == r2.getTuple(j)){
					duplicate = true;
					break;
				}
			}

			if (!duplicate){
				new_r.insertTuple(temp);
			}

			duplicate = false;
		}

		//make this one for loop later!

		for (int i = 0; i < r2.getAttributes()[0].getSize(); ++i) {
			temp = r2.getTuple(i);

			for (int j = 0; j < r1.getAttributes()[0].getSize(); ++j){
				if (temp == r1.getTuple(j)){
					duplicate = true;
					break;
				}
			}

			if (!duplicate){
				new_r.insertTuple(temp);
			}

			duplicate = false;
		}

		return new_r;
	}
}

Relation DBEngine::crossProduct(Relation r1, Relation r2){
	vector<Attribute> new_atts = r1.getAttributes();
	
	for (int i = 0; i < r2.getAttributes().size(); ++i) {
		new_atts.push_back(r2.getAttributes()[i]);
	}

	//currently all returned relations are called TEMP
	Relation new_r("TEMP", new_atts);
	

	return new_r;
}